'use strict';

const Base = require('./base');

class Project extends Base {
    /**
     * Get project
     *
     * @method GET
     * @uri /projects/{projectId}
     * @example seologs.project.get(projectId);
     *
     * @param {number} projectId the project ID.
     * @returns {Promise}
     */
    get(projectId) {
        return this.query({
            uri: '/projects/' + projectId,
        });
    }

    /**
     * List projects
     *
     * @method GET
     * @uri /projects
     * @example seologs.project.list(qs);
     *
     * @param {object} qs
     * @returns {Promise}
     */
    list(qs = {}) {
        return this.query({
            uri: '/projects',
            qs
        });
    }

    /**
     * Create project
     *
     * @method POST
     * @uri /projects
     * @example seologs.project.create(body);
     *
     * @param {object} body
     * @returns {Promise}
     *
     * ##### body:
     *
     * - `name`
     * - `host`
     * - `regex_format`
     * - `path`
     * - `database_region`
     * - `storage_region`
     * - `user_id`
     */
    create(body = {}) {
        return this.query({
            method: 'POST',
            uri: '/projects',
            body
        });
    }

    /**
     * Update project
     *
     * @method PUT
     * @uri /projects/{projectId}
     * @example seologs.project.update(projectId, body);
     *
     * @param {number} projectId the project ID.
     * @param {object} body
     * @returns {Promise}
     *
     * ##### body:
     *
     * - `name`
     * - `host`
     * - `regex_format`
     * - `path`
     * - `database_region`
     * - `storage_region`
     * - `user_id`
     */
    update(projectId, body = {}) {
        return this.query({
            method: 'PUT',
            uri: '/projects/' + projectId,
            body
        });
    }

    /**
     * Delete project
     *
     * @method DELETE
     * @uri /projects/{projectId}
     * @example seologs.project.delete(projectId);
     *
     * @param {number} projectId the project ID.
     * @returns {Promise}
     */
    delete(projectId) {
        return this.query({
            method: 'DELETE',
            uri: '/projects/' + projectId,
        });
    }
}

module.exports = Project;